@@ -11,13 +11,13 @@
 #include <QTabWidget>
 #include <QVBoxLayout>
 
+#include <KService>
 #include <kcmoduleinfo.h>
 #include <kcmoduleproxy.h>
-#include <KService>
 
 typedef QList<KCModuleProxy *> ModuleList;
 
-#if KCMUTILS_BUILD_DEPRECATED_SINCE(5,66)
+#if KCMUTILS_BUILD_DEPRECATED_SINCE(5, 66)
 
 /***********************************************************************/
 class Q_DECL_HIDDEN KCModuleContainer::KCModuleContainerPrivate
@@ -25,7 +25,8 @@ class Q_DECL_HIDDEN KCModuleContainer::KCModuleContainerPrivate
 public:
     KCModuleContainerPrivate(const QStringList &mods)
         : modules(mods)
-    {}
+    {
+    }
 
     QStringList modules;
     QTabWidget *tabWidget = nullptr;
@@ -42,26 +43,21 @@ public:
      * A list of all modules which are encapsulated.
      */
     ModuleList allModules;
-
 };
 /***********************************************************************/
 
 // The KCModuleContainer is only a wrapper around real KCModules.
 /***********************************************************************/
 KCModuleContainer::KCModuleContainer(QWidget *parent, const QString &mods)
-    : KCModule(parent),
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
-      d(new KCModuleContainerPrivate(QString(mods).remove(QLatin1Char(' ')).split(QLatin1Char(','), QString::SkipEmptyParts)))
-#else
-      d(new KCModuleContainerPrivate(QString(mods).remove(QLatin1Char(' ')).split(QLatin1Char(','), Qt::SkipEmptyParts)))
-#endif
+    : KCModule(parent)
+    , d(new KCModuleContainerPrivate(QString(mods).remove(QLatin1Char(' ')).split(QLatin1Char(','), Qt::SkipEmptyParts)))
 {
     init();
 }
 
 KCModuleContainer::KCModuleContainer(QWidget *parent, const QStringList &mods)
-    : KCModule(parent),
-      d(new KCModuleContainerPrivate(mods))
+    : KCModule(parent)
+    , d(new KCModuleContainerPrivate(mods))
 {
     init();
 }
@@ -106,13 +102,14 @@ void KCModuleContainer::addModule(const QString &module)
 
     proxy->setObjectName(module);
 
-    d->tabWidget->addTab(proxy, QIcon::fromTheme(proxy->moduleInfo().icon()),
+    d->tabWidget->addTab(proxy,
+                         QIcon::fromTheme(proxy->moduleInfo().icon()),
                          /* Qt eats ampersands for dinner. But not this time. */
                          proxy->moduleInfo().moduleName().replace(QLatin1Char('&'), QStringLiteral("&&")));
 
     d->tabWidget->setTabToolTip(d->tabWidget->indexOf(proxy), proxy->moduleInfo().comment());
 
-    connect(proxy, SIGNAL(changed(KCModuleProxy*)), SLOT(moduleChanged(KCModuleProxy*)));
+    connect(proxy, QOverload<KCModuleProxy *>::of(&KCModuleProxy::changed), this, &KCModuleContainer::moduleChanged);
 
     /* Collect our buttons - we go for the common deliminator */
     setButtons(buttons() | proxy->realModule()->buttons());
@@ -133,8 +130,7 @@ void KCModuleContainer::save()
         (*it)->save();
     }
 
-    emit changed(false);
-
+    Q_EMIT changed(false);
 }
 
 void KCModuleContainer::load()
@@ -145,7 +141,7 @@ void KCModuleContainer::load()
         (*it)->load();
     }
 
-    emit changed(false);
+    Q_EMIT changed(false);
 }
 
 void KCModuleContainer::defaults()
@@ -156,7 +152,7 @@ void KCModuleContainer::defaults()
         (*it)->defaults();
     }
 
-    emit changed(true);
+    Q_EMIT changed(true);
 }
 
 void KCModuleContainer::moduleChanged(KCModuleProxy *proxy)
@@ -166,7 +162,7 @@ void KCModuleContainer::moduleChanged(KCModuleProxy *proxy)
         return;
     }
 
-    emit changed(true);
+    Q_EMIT changed(true);
 }
 
 KCModuleContainer::~KCModuleContainer()
@@ -177,4 +173,3 @@ KCModuleContainer::~KCModuleContainer()
 #endif
 
 /***********************************************************************/
-
