@@ -3,6 +3,7 @@
     SPDX-FileCopyrightText: 1999 Matthias Hoelzer-Kluepfel <hoelzer@kde.org>
     SPDX-FileCopyrightText: 2002-2003 Daniel Molkentin <molkentin@kde.org>
     SPDX-FileCopyrightText: 2006 Matthias Kretz <kretz@kde.org>
+    SPDX-FileCopyrightText: 2021 Alexander Lohnau <alexander.lohnau@gmx.de>
 
     SPDX-License-Identifier: LGPL-2.0-only
 */
@@ -12,6 +13,7 @@
 
 #include <KCModule>
 #include <kcmoduleinfo.h>
+#include <kcmoduledata.h>
 
 class QWidget;
 
@@ -27,7 +29,7 @@ class QWidget;
  * @author Matthias Hoelzer-Kluepfel <mhk@kde.org>
  * @author Frans Englich <frans.englich@telia.com>
  * @internal
-**/
+ **/
 namespace KCModuleLoader
 {
 /**
@@ -50,7 +52,7 @@ enum ErrorReporting {
     /**
      * does both Inline and Dialog
      */
-    Both = 3
+    Both = 3,
 };
 
 /**
@@ -60,9 +62,16 @@ enum ErrorReporting {
  *
  * @return a pointer to the loaded @ref KCModule
  */
-KCMUTILS_EXPORT KCModule *loadModule(const KCModuleInfo &module, ErrorReporting
-                                     report, QWidget *parent = nullptr, const QStringList &args = QStringList());
+KCMUTILS_EXPORT KCModule *loadModule(const KCModuleInfo &module, ErrorReporting report, QWidget *parent = nullptr, const QStringList &args = QStringList());
 
+/**
+ * Loads a @ref KCModule. If loading fails a KCM which displays an error message is returned.
+
+ * @param metaData KPluginMetaData for loading the plugin
+ * @return a pointer to the loaded @ref KCModule
+ * @since 5.84
+ */
+KCMUTILS_EXPORT KCModule *loadModule(const KPluginMetaData &metaData, QWidget *parent = nullptr, const QVariantList &args = {});
 /**
  * Loads a @ref KCModule. If loading fails a zero pointer is returned.
  *
@@ -71,8 +80,7 @@ KCMUTILS_EXPORT KCModule *loadModule(const KCModuleInfo &module, ErrorReporting
  *
  * @return a pointer to the loaded @ref KCModule
  */
-KCMUTILS_EXPORT KCModule *loadModule(const QString &module, ErrorReporting report,
-                                     QWidget *parent = nullptr, const QStringList &args = QStringList());
+KCMUTILS_EXPORT KCModule *loadModule(const QString &module, ErrorReporting report, QWidget *parent = nullptr, const QStringList &args = QStringList());
 
 /**
  * Unloads the module's library
@@ -89,8 +97,7 @@ KCMUTILS_EXPORT void unloadModule(const KCModuleInfo &mod);
  *
  * @internal
  */
-KCMUTILS_EXPORT KCModule *reportError(ErrorReporting report, const QString &text,
-                                      const QString &details, QWidget *parent);
+KCMUTILS_EXPORT KCModule *reportError(ErrorReporting report, const QString &text, const QString &details, QWidget *parent);
 
 /**
  * For a specified module, return false if configuration is not the default one and true in all other case including if module is not found
@@ -101,6 +108,15 @@ KCMUTILS_EXPORT KCModule *reportError(ErrorReporting report, const QString &text
  * @since 5.74
  */
 KCMUTILS_EXPORT bool isDefaults(const KCModuleInfo &module, const QStringList &args = QStringList());
+
+/**
+ * Loads a @ref KCModuleData. If fails to load, return nullptr.
+ *
+ * @param module what module to load
+ *
+ * @since 5.81
+ */
+KCMUTILS_EXPORT KCModuleData *loadModuleData(const KCModuleInfo &module, const QStringList &args = QStringList());
 }
 
 #endif // KCMODULELOADER_H
