@@ -1,37 +1,25 @@
 /*
-   Copyright (c) 2000 Matthias Elter <elter@kde.org>
-   Copyright (c) 2003 Daniel Molkentin <molkentin@kde.org>
-   Copyright (c) 2003,2006 Matthias Kretz <kretz@kde.org>
-   Copyright (c) 2004 Frans Englich <frans.englich@telia.com>
-   Copyright (c) 2006 Tobias Koenig <tokoe@kde.org>
-
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-FileCopyrightText: 2000 Matthias Elter <elter@kde.org>
+    SPDX-FileCopyrightText: 2003 Daniel Molkentin <molkentin@kde.org>
+    SPDX-FileCopyrightText: 2003, 2006 Matthias Kretz <kretz@kde.org>
+    SPDX-FileCopyrightText: 2004 Frans Englich <frans.englich@telia.com>
+    SPDX-FileCopyrightText: 2006 Tobias Koenig <tokoe@kde.org>
+    SPDX-FileCopyrightText: 2021 Alexander Lohnau <alexander.lohnau@gmx.de>
 
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "kcmultidialog.h"
 #include "kcmultidialog_p.h"
 
-#include "kcmoduleqml_p.h"
 #include "kcmoduleproxy.h"
+#include "kcmoduleqml_p.h"
 #include <kcmutils_debug.h>
 
 #include <QApplication>
 #include <QDesktopServices>
 #include <QDesktopWidget>
+#include <QJsonArray>
 #include <QLayout>
 #include <QProcess>
 #include <QPushButton>
@@ -50,6 +38,7 @@
 #include <KMessageBox>
 #include <KPageWidgetModel>
 #include <KPluginInfo>
+#include <KPluginMetaData>
 
 bool KCMultiDialogPrivate::resolveChanges(KCModuleProxy *currentProxy)
 {
@@ -59,14 +48,13 @@ bool KCMultiDialogPrivate::resolveChanges(KCModuleProxy *currentProxy)
     }
 
     // Let the user decide
-    const int queryUser = KMessageBox::warningYesNoCancel(
-                              q,
-                              i18n("The settings of the current module have changed.\n"
-                                   "Do you want to apply the changes or discard them?"),
-                              i18n("Apply Settings"),
-                              KStandardGuiItem::apply(),
-                              KStandardGuiItem::discard(),
-                              KStandardGuiItem::cancel());
+    const int queryUser = KMessageBox::warningYesNoCancel(q,
+                                                          i18n("The settings of the current module have changed.\n"
+                                                               "Do you want to apply the changes or discard them?"),
+                                                          i18n("Apply Settings"),
+                                                          KStandardGuiItem::apply(),
+                                                          KStandardGuiItem::discard(),
+                                                          KStandardGuiItem::cancel());
 
     switch (queryUser) {
     case KMessageBox::Yes:
@@ -96,8 +84,26 @@ void KCMultiDialogPrivate::_k_slotCurrentPageChanged(KPageWidgetItem *current, K
         }
     }
 
-    // Remove margins for the buttonbox; KPageDialog handles that already
-    q->buttonBox()->setContentsMargins(0, 0, 0, 0);
+    // Delete global margins and spacing, since we want the contents to
+    // be able to touch the edges of the window
+    q->layout()->setContentsMargins(0, 0, 0, 0);
+
+    const KPageWidget *pageWidget = q->pageWidget();
+    pageWidget->layout()->setSpacing(0);
+
+    // Then, we set the margins for the title header and the buttonBox footer
+    const QStyle *style = q->style();
+    const QMargins layoutMargins = QMargins(style->pixelMetric(QStyle::PM_LayoutLeftMargin),
+                                            style->pixelMetric(QStyle::PM_LayoutTopMargin),
+                                            style->pixelMetric(QStyle::PM_LayoutRightMargin),
+                                            style->pixelMetric(QStyle::PM_LayoutBottomMargin));
+
+    if (pageWidget->pageHeader()) {
+        pageWidget->pageHeader()->setContentsMargins(layoutMargins);
+    }
+
+    // Do not set buttonBox's top margin as that space will be covered by the content's bottom margin
+    q->buttonBox()->setContentsMargins(layoutMargins.left(), 0, layoutMargins.right(), layoutMargins.bottom());
 
     q->blockSignals(true);
     q->setCurrentPage(previous);
@@ -118,8 +124,8 @@ void KCMultiDialogPrivate::_k_clientChanged()
     // Get the current module
     KCModuleProxy *activeModule = nullptr;
     for (int i = 0; i < modules.count(); ++i) {
-        if (modules[ i ].item == q->currentPage()) {
-            activeModule = modules[ i ].kcm;
+        if (modules[i].item == q->currentPage()) {
+            activeModule = modules[i].kcm;
             break;
         }
     }
@@ -223,9 +229,9 @@ void KCMultiDialogPrivate::_k_updateHeader(bool use, const QString &message)
     KCModuleProxy *kcm = qobject_cast<KCModuleProxy *>(item->widget());
 
     if (use) {
-        item->setHeader(QStringLiteral("<b>") + kcm->moduleInfo().moduleName() + QStringLiteral("</b><br><i>") +
-                        message + QStringLiteral("</i>"));
-        item->setIcon(KIconUtils::addOverlay(QIcon::fromTheme(kcm->moduleInfo().icon()), QIcon::fromTheme(QStringLiteral("dialog-warning")), Qt::BottomRightCorner));
+        item->setHeader(QStringLiteral("<b>") + kcm->moduleInfo().moduleName() + QStringLiteral("</b><br><i>") + message + QStringLiteral("</i>"));
+        item->setIcon(
+            KIconUtils::addOverlay(QIcon::fromTheme(kcm->moduleInfo().icon()), QIcon::fromTheme(QStringLiteral("dialog-warning")), Qt::BottomRightCorner));
     } else {
         item->setHeader(kcm->moduleInfo().moduleName());
         item->setIcon(QIcon::fromTheme(kcm->moduleInfo().icon()));
@@ -240,13 +246,8 @@ void KCMultiDialogPrivate::init()
     q->setModal(false);
 
     QDialogButtonBox *buttonBox = new QDialogButtonBox(q);
-    buttonBox->setStandardButtons(QDialogButtonBox::Help
-                                  | QDialogButtonBox::RestoreDefaults
-                                  | QDialogButtonBox::Cancel
-                                  | QDialogButtonBox::Apply
-                                  | QDialogButtonBox::Close
-                                  | QDialogButtonBox::Ok
-                                  | QDialogButtonBox::Reset);
+    buttonBox->setStandardButtons(QDialogButtonBox::Help | QDialogButtonBox::RestoreDefaults | QDialogButtonBox::Cancel | QDialogButtonBox::Apply
+                                  | QDialogButtonBox::Close | QDialogButtonBox::Ok | QDialogButtonBox::Reset);
     KGuiItem::assign(buttonBox->button(QDialogButtonBox::Ok), KStandardGuiItem::ok());
     KGuiItem::assign(buttonBox->button(QDialogButtonBox::Cancel), KStandardGuiItem::cancel());
     KGuiItem::assign(buttonBox->button(QDialogButtonBox::RestoreDefaults), KStandardGuiItem::defaults());
@@ -265,9 +266,9 @@ void KCMultiDialogPrivate::init()
     q->connect(buttonBox->button(QDialogButtonBox::Reset), &QAbstractButton::clicked, q, &KCMultiDialog::slotUser1Clicked);
 
     q->setButtonBox(buttonBox);
-
-    q->connect(q, SIGNAL(currentPageChanged(KPageWidgetItem*,KPageWidgetItem*)),
-               SLOT(_k_slotCurrentPageChanged(KPageWidgetItem*,KPageWidgetItem*)));
+    q->connect(q, &KPageDialog::currentPageChanged, q, [this](KPageWidgetItem *current, KPageWidgetItem *before) {
+        _k_slotCurrentPageChanged(current, before);
+    });
 }
 
 KCMultiDialog::KCMultiDialog(QWidget *parent)
@@ -310,8 +311,7 @@ void KCMultiDialog::showEvent(QShowEvent *ev)
     QScreen *screen = QApplication::screenAt(pos());
     if (screen) {
         const QSize maxSize = screen->availableGeometry().size();
-        resize(qMin(sizeHint().width(), maxSize.width()),
-                qMin(sizeHint().height(), maxSize.height()));
+        resize(qMin(sizeHint().width(), maxSize.width()), qMin(sizeHint().height(), maxSize.height()));
     }
 }
 
@@ -324,8 +324,8 @@ void KCMultiDialog::slotDefaultClicked()
     }
 
     for (int i = 0; i < d->modules.count(); ++i) {
-        if (d->modules[ i ].item == item) {
-            d->modules[ i ].kcm->defaults();
+        if (d->modules[i].item == item) {
+            d->modules[i].kcm->defaults();
             d->_k_clientChanged();
             return;
         }
@@ -341,8 +341,8 @@ void KCMultiDialog::slotUser1Clicked()
 
     Q_D(KCMultiDialog);
     for (int i = 0; i < d->modules.count(); ++i) {
-        if (d->modules[ i ].item == item) {
-            d->modules[ i ].kcm->load();
+        if (d->modules[i].item == item) {
+            d->modules[i].kcm->load();
             d->_k_clientChanged();
             return;
         }
@@ -370,9 +370,9 @@ void KCMultiDialogPrivate::apply()
         if (proxy->changed()) {
             proxy->save();
             /**
-                * Add name of the components the kcm belongs to the list
-                * of updated components.
-                */
+             * Add name of the components the kcm belongs to the list
+             * of updated components.
+             */
             const QStringList componentNames = module.componentNames;
             for (const QString &componentName : componentNames) {
                 if (!updatedComponents.contains(componentName)) {
@@ -384,10 +384,10 @@ void KCMultiDialogPrivate::apply()
 
     // Send the configCommitted signal for every updated component.
     for (const QString &name : qAsConst(updatedComponents)) {
-        emit q->configCommitted(name.toLatin1());
+        Q_EMIT q->configCommitted(name.toLatin1());
     }
 
-    emit q->configCommitted();
+    Q_EMIT q->configCommitted();
 }
 
 void KCMultiDialog::slotApplyClicked()
@@ -417,8 +417,11 @@ void KCMultiDialog::slotHelpClicked()
     Q_D(KCMultiDialog);
     QString docPath;
     for (int i = 0; i < d->modules.count(); ++i) {
-        if (d->modules[ i ].item == item) {
-            docPath = d->modules[ i ].kcm->moduleInfo().docPath();
+        if (d->modules[i].item == item) {
+            docPath = d->modules[i].kcm->moduleInfo().docPath();
+            if (docPath.isEmpty()) {
+                docPath = d->modules[i].kcm->metaData().value(QStringLiteral("X-DocPath"));
+            }
             break;
         }
     }
@@ -442,11 +445,84 @@ void KCMultiDialog::closeEvent(QCloseEvent *event)
      * in other situations will lead to "module already loaded in Foo," while to the user
      * doesn't appear so(the dialog is hidden)
      */
-    for(auto &proxy : qAsConst(d->modules)) {
+    for (auto &proxy : qAsConst(d->modules)) {
         proxy.kcm->deleteClient();
     }
 }
 
+KPageWidgetItem *KCMultiDialog::addModule(const KPluginMetaData &metaData)
+{
+    Q_D(KCMultiDialog);
+    // Create the scroller
+    auto *moduleScroll = new UnboundScrollArea(this);
+    // Prepare the scroll area
+    moduleScroll->setWidgetResizable(true);
+    moduleScroll->setFrameStyle(QFrame::NoFrame);
+    moduleScroll->viewport()->setAutoFillBackground(false);
+
+    KCModuleProxy *kcm = new KCModuleProxy(metaData, moduleScroll, QStringList());
+    moduleScroll->setWidget(kcm);
+
+    KPageWidgetItem *item = new KPageWidgetItem(moduleScroll, metaData.name());
+
+    KCMultiDialogPrivate::CreatedModule createdModule;
+    createdModule.kcm = kcm;
+    createdModule.item = item;
+    d->modules.append(createdModule);
+
+    if (qobject_cast<KCModuleQml *>(kcm->realModule())) {
+        item->setHeaderVisible(false);
+    }
+
+    if (kcm->realModule() && kcm->realModule()->useRootOnlyMessage()) {
+        item->setHeader(QStringLiteral("<b>%1</b><br><i>%2</i>").arg(metaData.name(), kcm->realModule()->rootOnlyMessage()));
+        item->setIcon(KIconUtils::addOverlay(QIcon::fromTheme(metaData.iconName()), QIcon::fromTheme(QStringLiteral("dialog-warning")), Qt::BottomRightCorner));
+    } else {
+        item->setHeader(metaData.name());
+        item->setIcon(QIcon::fromTheme(metaData.iconName()));
+    }
+    const int weight = metaData.rawData().value(QStringLiteral("X-KDE-Weight")).toInt();
+    item->setProperty("_k_weight", weight);
+
+    bool updateCurrentPage = false;
+    const KPageWidgetModel *model = qobject_cast<const KPageWidgetModel *>(pageWidget()->model());
+    Q_ASSERT(model);
+    const int siblingCount = model->rowCount();
+    int row = 0;
+    for (; row < siblingCount; ++row) {
+        KPageWidgetItem *siblingItem = model->item(model->index(row, 0));
+        if (siblingItem->property("_k_weight").toInt() > weight) {
+            // the item we found is heavier than the new module
+            // qDebug() << "adding KCM " << item->name() << " before " << siblingItem->name();
+            insertPage(siblingItem, item);
+            if (siblingItem == currentPage()) {
+                updateCurrentPage = true;
+            }
+
+            break;
+        }
+    }
+    if (row == siblingCount) {
+        // the new module is either the first or the heaviest item
+        // qDebug() << "adding KCM " << item->name() << " at the top level";
+        addPage(item);
+    }
+
+    QObject::connect(kcm, QOverload<bool>::of(&KCModuleProxy::changed), this, [d]() {
+        d->_k_clientChanged();
+    });
+
+    QObject::connect(kcm->realModule(), &KCModule::rootOnlyMessageChanged, this, [d](bool use, const QString &message) {
+        d->_k_updateHeader(use, message);
+    });
+
+    if (d->modules.count() == 1 || updateCurrentPage) {
+        setCurrentPage(item);
+        d->_k_clientChanged();
+    }
+    return item;
+}
+
 KPageWidgetItem *KCMultiDialog::addModule(const QString &path, const QStringList &args)
 {
     QString complete = path;
@@ -460,16 +536,15 @@ KPageWidgetItem *KCMultiDialog::addModule(const QString &path, const QStringList
     return addModule(KCModuleInfo(service), nullptr, args);
 }
 
-KPageWidgetItem *KCMultiDialog::addModule(const KCModuleInfo &moduleInfo,
-        KPageWidgetItem *parentItem, const QStringList &args)
+KPageWidgetItem *KCMultiDialog::addModule(const KCModuleInfo &moduleInfo, KPageWidgetItem *parentItem, const QStringList &args)
 {
     Q_D(KCMultiDialog);
     if (!moduleInfo.isValid()) {
         return nullptr;
     }
 
-    //KAuthorized::authorizeControlModule( moduleInfo.service()->menuId() ) is
-    //checked in noDisplay already
+    // KAuthorized::authorizeControlModule( moduleInfo.service()->menuId() ) is
+    // checked in noDisplay already
     if (moduleInfo.service() && moduleInfo.service()->noDisplay()) {
         return nullptr;
     }
@@ -498,11 +573,8 @@ KPageWidgetItem *KCMultiDialog::addModule(const KCModuleInfo &moduleInfo,
     }
 
     if (kcm->realModule() && kcm->realModule()->useRootOnlyMessage()) {
-        item->setHeader(QStringLiteral("<b>") +
-                moduleInfo.moduleName() +
-                QStringLiteral("</b><br><i>") +
-                kcm->realModule()->rootOnlyMessage() +
-                QStringLiteral("</i>"));
+        item->setHeader(QStringLiteral("<b>") + moduleInfo.moduleName() + QStringLiteral("</b><br><i>") + kcm->realModule()->rootOnlyMessage()
+                        + QStringLiteral("</i>"));
         item->setIcon(KIconUtils::addOverlay(QIcon::fromTheme(moduleInfo.icon()), QIcon::fromTheme(QStringLiteral("dialog-warning")), Qt::BottomRightCorner));
     } else {
         item->setHeader(moduleInfo.moduleName());
@@ -554,8 +626,13 @@ KPageWidgetItem *KCMultiDialog::addModule(const KCModuleInfo &moduleInfo,
         }
     }
 
-    connect(kcm, SIGNAL(changed(bool)), this, SLOT(_k_clientChanged()));
-    connect(kcm->realModule(), SIGNAL(rootOnlyMessageChanged(bool,QString)), this, SLOT(_k_updateHeader(bool,QString)));
+    QObject::connect(kcm, QOverload<bool>::of(&KCModuleProxy::changed), this, [d]() {
+        d->_k_clientChanged();
+    });
+
+    QObject::connect(kcm->realModule(), &KCModule::rootOnlyMessageChanged, this, [d](bool use, QString message) {
+        d->_k_updateHeader(use, message);
+    });
 
     if (d->modules.count() == 1 || updateCurrentPage) {
         setCurrentPage(item);
@@ -570,8 +647,8 @@ void KCMultiDialog::clear()
     // qDebug() ;
 
     for (int i = 0; i < d->modules.count(); ++i) {
-        removePage(d->modules[ i ].item);
-        delete d->modules[ i ].kcm;
+        removePage(d->modules[i].item);
+        delete d->modules[i].kcm;
     }
 
     d->modules.clear();
